# test_tofseries.py --- 
# 
# Filename: test_tofseries.py
# Description: 
#            unittests of tofseries
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Sat Jul 21 07:25:44 2018 (-0500)
# Version: 
# Last-Updated: Tue Jul 31 22:30:18 2018 (-0500)
#           By: yulu
#     Update #: 87
# 

import unittest
import pkg_resources
import numpy as np 
from scibeam import TOFSeries, read_file


DATA_FILE = pkg_resources.resource_filename('scibeam', 'data/test/time_series_1D/single_time_series.lvm')


class TestFunctions(unittest.TestCase):
    
    def test_read_defaults(self):
        try:
            ds = read_file(DATA_FILE)
        except:
            self.fail("File read-in failed !")
        else:
            self.assertEqual(ds.shape, (25000, ))
            self.assertTrue(isinstance(ds, TOFSeries))
        
    def test_read_with_bounds(self):
        ds = read_file(DATA_FILE, lowerBound = 500e-6, upperBound = 600e-6)
        try:
            ds = read_file(DATA_FILE, lowerBound = 500e-6, upperBound = 600e-6)
        except:
            self.fail("File read-in failed !")
        self.assertTrue(abs(ds.index[0] - 500e-6) < 1e-6)
        self.assertTrue(abs(ds.index[-1] - 600e-6) < 1e-6)

    def test_read_without_offset(self):
        try:
            ds = read_file(DATA_FILE, lowerBound = 500e-6, upperBound = 600e-6, removeOffset = False)
            ds2 = read_file(DATA_FILE, lowerBound = 500e-6, upperBound = 600e-6, removeOffset = True)
        except:
            self.fail("File read-in failed !")
        self.assertTrue(abs(ds - ds2).sum() > 0.01)

        
class TestTOFSeries(unittest.TestCase):

    def setUp(self):
        try:
            self.series = TOFSeries.from_file(DATA_FILE)
        except:
            self.fail("TOFSeries initialize failed !")

    def test_init(self):
        self.assertTrue(hasattr(self.series, 'head'))

    def test_find_time_idx(self):
        self.assertEqual(list(self.series.find_time_idx(self.series.index, 1000e-6)), [2500])
        self.assertEqual(list(self.series.find_time_idx(self.series.index, 1000e-6, 1100e-6, 1200e-6)), [2500, 2750, 3000])
        self.assertEqual(list(self.series.find_time_idx(self.series.index, [1000e-6, 1100e-6, 1200e-6])), [2500, 2750, 3000])

    def test_remove_data_offset(self):
        sample_data = np.ones(1000)
        sample_data[450:550] = 10
        sample_data_offset_removed1 = TOFSeries.remove_data_offset(sample_data,
                                                                  lowerBoundIdx = 400,
                                                                  upperBoundIdx = 600)
        sample_data_offset_removed2 = TOFSeries.remove_data_offset(sample_data)
        
        self.assertTrue((sample_data_offset_removed1[450:550] == 9).any())
        self.assertTrue((sample_data_offset_removed1[:200] == 0).any())
        self.assertTrue((sample_data_offset_removed2[450:550] == 9).any())
        self.assertTrue((sample_data_offset_removed2[:200] == 0).any())

    def test_selectTimeSlice(self):
        self.assertEqual(self.series.selectTimeSlice(1000e-6)[0], -0.001125382)
        self.assertTrue((np.array(self.series.selectTimeSlice(1000e-6, 1100e-6))
                         - np.array([-0.001125382, 0.002536727])
                         == 0).all())

        self.assertEqual(list(self.series.selectTimeSlice([1000e-6, 1100e-6])),
                         [-0.001125382, 0.002536727])

        
        
        
                         
                

if __name__ == '__main__':
    unittest.main()



# test_formatter.py --- 
# 
# Filename: test_formatter.py
# Description: 
#            Unittests for formatter functions 
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Thu Jul 19 11:26:04 2018 (-0500)
# Version: 
# Last-Updated: Tue Jul 24 23:58:06 2018 (-0500)
#           By: yulu
#     Update #: 15
# 

import unittest
from scibeam.core.formatter import format_dict

class TestFunctions(unittest.TestCase):
    testDict = {'d': 4,
                'c': 3.1415926,
                'e': 5,
                'b': 2,
                'a': 1
                }

    def test_formart_dict(self):
        self.assertEqual(format_dict(self.testDict, alphabetical = False, digits = 2),
                         'd: 4.00\nc: 3.14\ne: 5.00\nb: 2.00\na: 1.00')

        self.assertEqual(format_dict(self.testDict, alphabetical = True, digits = 2),
                         'a: 1.00\nb: 2.00\nc: 3.14\nd: 4.00\ne: 5.00')
        self.assertEqual(format_dict(self.testDict, alphabetical = True, digits = 5),
                         'a: 1.00000\nb: 2.00000\nc: 3.14159\nd: 4.00000\ne: 5.00000')


if __name__ == '__main__':
    unittest.main()
                         



# test_regexp.py --- 
# 
# Filename: test_regexp.py
# Description: 
# 
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Wed Jul 25 00:17:55 2018 (-0500)
# Version: 
# Last-Updated: Sat Jul 28 15:04:02 2018 (-0500)
#           By: yulu
#     Update #: 59
# 

import unittest
import pkg_resources
import re
from scibeam import RegMatch



DATA_PATH = pkg_resources.resource_filename('scibeam', 'data/test/time_series_2D/')

class TestRegmatch(unittest.TestCase):
    def setUp(self):
        try:
            self.rgm_single = RegMatch('.*_(\d+\.\d+)in_.*.lvm$')
            self.rgm_multi = RegMatch(['.*_(APD\d+)_.*.lvm$', '.*_(\d+\.\d+)in_.*.lvm$'])
        except:
            self.fail("Initialize class RegMatch instance failed !")
        else:
            self.assertTrue(hasattr(self.rgm_single.regex, 'match'))
            self.assertTrue(hasattr(self.rgm_multi.regex, '__iter__'))
            
    def test_single_regex_match(self):
        sample = ['20180314_APD1_0.77500in_scan.lvm', '20180314_APD1_0.92500in_scan.lvm']
        regex = '.*(APD\d+)_(\d+\.\d+)in_.*.lvm$'

        # Test when being matched is a single string 
        self.assertEqual(self.rgm_single.single_regex_match(regex, sample[0], group = 1, asNumber = False),
                         {'APD1': '20180314_APD1_0.77500in_scan.lvm'})
        
        self.assertEqual(self.rgm_single.single_regex_match(regex, sample[0], group = 2, asNumber = False),
                         {'0.77500': '20180314_APD1_0.77500in_scan.lvm'})
        
        self.assertEqual(self.rgm_single.single_regex_match(regex, sample[0], group = 2, asNumber = True),
                         {0.775: '20180314_APD1_0.77500in_scan.lvm'})

        # Test when being matched is a list of strings
        self.assertEqual(self.rgm_single.single_regex_match(regex, sample, group = 1, asNumber = False),
                         {'APD1': ['20180314_APD1_0.77500in_scan.lvm', '20180314_APD1_0.92500in_scan.lvm']})
        self.assertEqual(self.rgm_single.single_regex_match(regex, sample, group = 2, asNumber = False),
                         {'0.77500': '20180314_APD1_0.77500in_scan.lvm', '0.92500':  '20180314_APD1_0.92500in_scan.lvm'})
        self.assertEqual(self.rgm_single.single_regex_match(regex, sample, group = 2, asNumber = True),
                         {0.775: '20180314_APD1_0.77500in_scan.lvm', 0.925: '20180314_APD1_0.92500in_scan.lvm'})

        
    def test_match(self):
        sample = ['20180314_APD1_0.77500in_scan.lvm', '20180314_APD1_0.92500in_scan.lvm']

        # one to one
        self.assertEqual(self.rgm_single.match(sample[0], group = 1, asNumber = True),
                         {0.775: '20180314_APD1_0.77500in_scan.lvm'})
        self.assertEqual(self.rgm_single.match(sample[0], group = 1, asNumber = False),
                         {'0.77500': '20180314_APD1_0.77500in_scan.lvm'})

        # one to multi 
        self.assertEqual(self.rgm_single.match(sample, group = 1, asNumber = True),
                         {0.775: '20180314_APD1_0.77500in_scan.lvm',
                          0.925: '20180314_APD1_0.92500in_scan.lvm'})
        self.assertEqual(self.rgm_single.match(sample, group = 1, asNumber = False),
                         {'0.77500': '20180314_APD1_0.77500in_scan.lvm',
                          '0.92500': '20180314_APD1_0.92500in_scan.lvm'})

        # multi to one
        self.assertEqual(self.rgm_multi.match(sample[0], group = 1, asNumber = False),
                         [{'APD1': '20180314_APD1_0.77500in_scan.lvm'},
                          {'0.77500': '20180314_APD1_0.77500in_scan.lvm'}])

        # multi to multi
        self.assertEqual(self.rgm_multi.match(sample, group = 1, asNumber = False),
                         [{'APD1': ['20180314_APD1_0.77500in_scan.lvm',
                                    '20180314_APD1_0.92500in_scan.lvm']},
                          {'0.77500': '20180314_APD1_0.77500in_scan.lvm',
                           '0.92500': '20180314_APD1_0.92500in_scan.lvm'}])


    def test_matchFolder(self):
        self.assertEqual(self.rgm_single.matchFolder(DATA_PATH, asNumber = True),
        {0.3: '20180314_APD1_0.30000in_scan.lvm',
         0.325: '20180314_APD1_0.32500in_scan.lvm',
         0.35: '20180314_APD1_0.35000in_scan.lvm',
         0.375: '20180314_APD1_0.37500in_scan.lvm',
         0.4: '20180314_APD1_0.40000in_scan.lvm'})

        self.assertEqual(self.rgm_single.matchFolder(DATA_PATH, asNumber = False),
        {'0.30000': '20180314_APD1_0.30000in_scan.lvm',
         '0.32500': '20180314_APD1_0.32500in_scan.lvm',
         '0.35000': '20180314_APD1_0.35000in_scan.lvm',
         '0.37500': '20180314_APD1_0.37500in_scan.lvm',
         '0.40000': '20180314_APD1_0.40000in_scan.lvm'})


        self.assertEqual(self.rgm_multi.matchFolder(DATA_PATH, asNumber = False),
        {'APD1':
         {'0.30000': '20180314_APD1_0.30000in_scan.lvm',
         '0.32500': '20180314_APD1_0.32500in_scan.lvm',
         '0.35000': '20180314_APD1_0.35000in_scan.lvm',
         '0.37500': '20180314_APD1_0.37500in_scan.lvm',
         '0.40000': '20180314_APD1_0.40000in_scan.lvm'}
        })
        
        



# test_core_common.py --- 
# 
# Filename: test_core_common.py
# Description: 
# 
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Fri May  4 11:33:37 2018 (-0500)
# Version: 
# Last-Updated: Sat Jul 28 15:00:42 2018 (-0500)
#           By: yulu
#     Update #: 24
# 

import unittest
import os
import numpy as np 
#test_data_root = '../examples/data/'

from scibeam.core.common  import winPathHandler, loadFile
import pkg_resources

DATA_FILE = pkg_resources.resource_filename('scibeam', 'data/test/time_series_1D/single_time_series.lvm')
    


class TestFunctions(unittest.TestCase):
    '''
    Test core.common.py
    '''

    def test_winPathHandler(self):
        test_path_win = r'C:\Documents\MyFolder\Whatever.txt'
        test_path_linux = '/home/MyFolder/Whatever.txt'
        test_folder_linux = '../examples/data'

        self.assertEqual(winPathHandler(test_path_win), 'C:/Documents/MyFolder/Whatever.txt')
        self.assertEqual(winPathHandler(test_path_linux), '/home/MyFolder/Whatever.txt')
        self.assertEqual(winPathHandler([test_path_win, test_path_linux]),['C:/Documents/MyFolder/Whatever.txt','/home/MyFolder/Whatever.txt'])
        self.assertEqual(winPathHandler(test_folder_linux), '../examples/data')
    

    def test_loadFile(self):
        self.assertEqual(loadFile(DATA_FILE).shape, (25000, 2))
        
if __name__ == '__main__':
    unittest.main()

        



# test_base.py --- 
# 
# Filename: test_base.py
# Description:
#           Unit test case for base.py
# 
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Thu Mar 29 10:44:57 2018 (-0500)
# Version: 
# Last-Updated: Tue Jul 24 23:49:48 2018 (-0500)
#           By: yulu
#     Update #: 45
# 

import unittest
from scibeam.core.base import _is_mixin, _mixin_class
from scibeam import TOFFrame, TOFSeries, Gaussian

class TestFunctions(unittest.TestCase):
    def setUp(self):
        self.tofseries = TOFSeries()
        self.tofframe = TOFFrame()
        self.gaussian = Gaussian
        
    def test_is_mixin(self):
        self.assertTrue(_is_mixin(self.tofseries))
        self.assertTrue(_is_mixin(self.tofframe))
        self.assertFalse(_is_mixin(self.gaussian))
        
if __name__ == '__main__':
    unittest.main()



# __init__.py --- 
# 
# Filename: __init__.py
# Description: 
# 
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Fri May  4 11:14:42 2018 (-0500)
# Version: 
# Last-Updated: Tue Jul 24 23:49:37 2018 (-0500)
#           By: yulu
#     Update #: 8
# 





# __init__.py --- 
# 
# Filename: __init__.py
# Description:
#            Module scibeam __init__
# 
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Sun Mar 25 16:04:08 2018 (-0500)
# Version: 
# Last-Updated: Sun Aug 19 15:03:24 2018 (-0500)
#           By: yulu
#     Update #: 53
# 

"""Scibeam
For scientific time series analysis and visualization.


"""

name = "scibeam"


from scibeam.core.tofframe import (
    TOFFrame,
    read_folder,
    read_regexp_match,
    )

from scibeam.core.tofseries import (
    TOFSeries,
    read_file,
    )

from scibeam.core.plot import (
    PlotTOFSeries,
    PlotTOFFrame,
    )

from scibeam.core.regexp import RegMatch

from scibeam.core.gaussian import Gaussian





# test_import.py --- 
# 
# Filename: test_import.py
# Description: 
#            unittests for __init__ import  
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Sat Jul 21 07:07:02 2018 (-0500)
# Version: 
# Last-Updated: Tue Jul 24 23:52:33 2018 (-0500)
#           By: yulu
#     Update #: 17
# 

import unittest
import scibeam

class TestImports(unittest.TestCase):
    
            
    def test_import_TOFFrame(self):
        self.assertEqual(scibeam.TOFFrame, scibeam.core.tofframe.TOFFrame)

    def test_import_TOFSeries(self):
        self.assertEqual(scibeam.TOFSeries, scibeam.core.tofseries.TOFSeries)
        
    def test_import_read_folder(self):
        self.assertEqual(scibeam.read_folder, scibeam.core.tofframe.read_folder)

    def test_import_read_file(self):
        self.assertEqual(scibeam.read_file, scibeam.core.tofseries.read_file)

    def test_import_RegMatch(self):
        self.assertEqual(scibeam.RegMatch, scibeam.core.regexp.RegMatch)

    def test_import_PlotTOFSeries(self):
        self.assertEqual(scibeam.PlotTOFSeries, scibeam.core.plot.PlotTOFSeries)

    def test_import_Gaussian(self):
        self.assertEqual(scibeam.Gaussian, scibeam.core.gaussian.Gaussian)
    

if __name__ == '__main__':
    unittest.main()



# setup.py --- 
# 
# Filename: setup.py
# Description: 
# 
# Author:    Yu Lu
# Email:     yulu@utexas.edu
# Github:    https://github.com/SuperYuLu 
# 
# Created: Sat Jul 21 10:46:04 2018 (-0500)
# Version: 
# Last-Updated: Sun Aug 19 18:12:09 2018 (-0500)
#           By: yulu
#     Update #: 71
# 

from setuptools import setup

with open("README.rst") as f:
    README = f.read()

setup(
    name = 'scibeam',
    version = '0.1.0',
    author = 'Yu Lu',
    author_email = 'yulu@utexas.edu',
    url = 'https://github.com/SuperYuLu/SciBeam',
    packages = ['scibeam', 'scibeam.tests'],
    license = 'LICENSE.txt',
    description = 'A scientific time series analyzing tool on beam measurement',
    long_description = README,
    #long_description_content_type='text/markdown',
    install_requires = ['numpy',
                        'pandas',
                        'scipy',
                        'matplotlib'],
    
    #test_suite = 'nose.collector',
    #test_suite = 'tests',
    #tests_require = ['nose'],

    classifiers = [
        'Programming Language :: Python :: 3.6',
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Physics',
        'License :: OSI Approved :: MIT License',
        ],
    
    keywords = 'physics time-series data-analysis pandas',
    
    package_data = {'scibeam': ['data/examples/time_series_1D/single_time_series.lvm',
                                'data/examples/time_series_2D/*.lvm',
                                'data/test/time_series_1D/single_time_series.lvm',
                                'data/test/time_series_2D/*.lvm'
    ]},
)
